; simple LCD interface

;
; define PIC device type
;
	device	pic16f84

;
; define config fuses
;
	config	CP=off,WDT=off,PWRT=off,OSC=rc

;
; include PIC register definitions, and some macros
;
	include "pic16c84.h"
	include "picmac.h"

;
;                        -----------
;                       |     U     |
;               HD_E  --|RA2     RA1|-- HD_R/W
;                     --|RA3     RA0|-- HD_RS
;           SYNC DATA --|RA4        |
;                       |           | 
;                       |           |
;                       |           |
;                     --|RB0     RB7|-- D7
;                     --|RB1     RB6|-- D6
;                     --|RB2     RB5|-- D5
;                     --|RB3     RB4|-- D4
;                       |           |
;                        -----------



;
; bit definitions for two LEDs connected to port A bits 0 and 1
; bit masks can be computed from bit numbers with the left shift operator.
;

#define DEVICE_ID 1

#define HD_RS PORTA, 0
#define HD_RW PORTA, 1
#define HD_E PORTA, 2

#define LED PORTB, 1

#define SYNC_CLOCK PORTB, 7
#define SYNC_DATA PORTA,3
#define SYNC_RECEIVED ipc, 0
;
; define some register file variables
;

	org	0x0c


delay_cnt1	ds	1
delay_cnt2	ds	1
temp ds 1
temp_w ds 1
temp_s ds 1

lcd_temp ds 1
numdisp_temp ds 1
table_index ds 1

count_low ds 1
count ds 1

period ds 1
oldb ds 1
newb ds 1

brightness ds 1
brightness2 ds 1
brightness3 ds 1
brightness4 ds 1
brightness5 ds 1
brightness6 ds 1
direction ds 1

;
; code start
;
	org	0
	goto start

	org 4
	save_w_stat

	restore_w_stat
 	retfie


LabelTable
	addwf	PCL,F
LabelBase
Label0
	retlw 'L'
	retlw 'C'
	retlw 'D'
	retlw ' '
	retlw ' '
	retlw 0
Label1
	retlw 'T'
	retlw 'e'
	retlw 's'
	retlw 't'
	retlw ' '
	retlw 0
LabelTable_End
	retlw 0

Num_Table
	andlw	0x0f
Num_Tbl
	addwf	PCL,F
	retlw '0'
	retlw '1'
	retlw '2'
	retlw '3'
	retlw '4'
	retlw '5'
	retlw '6'
	retlw '7'
	retlw '8'
	retlw '9'
	retlw 'A'
	retlw 'B'
	retlw 'C'
	retlw 'D'
	retlw 'E'
	retlw 'F'

	
;org 0x100
start
	movlw	0xff
	call	delay

	movlw	0x00
	movwf	PORTA	;initialize port A so that LEDs are off

	movlw	0x00
	movwf	PORTB
	bsf	LED

	bsf	STATUS,RP0			;register page 1
	movlw	b'00001000'			;a4 input, all others output
	movwf	TRISA				;other PORTA pins as inputs
	movlw	b'00000001'			;b0 input, all others output
	movwf	TRISB

	; set options
	; 7 : 1 PORTB pull-ups are disabled
	; 6 : 1 Interrupt on rising esdge of RB0
	; 5 : 0 TMR0 on internal clock
	; 4 : 1 increment on high to low transition at RA4 for external clock
	; 3 : 0 prescaler assigned to TMR0
	; 2-0 : 101  1:64 timer prescale
	movlw	b'11010101'
	movwf	OPTIO

	bcf	STATUS,RP0			;register page 0


; turn off timer overflow interrupt	
	bcf INTCON,RTIE

; turn off RB0 edge interrupt
	bcf INTCON,INTE

; globally disable interrupts
	bcf	INTCON,GIE

	movlw	50
	call delay

	call init_lcd

	movlw	Label0-LabelBase
	call	Print_Label

	movlw	0x40
	call	LCD_SDDAM

	movlw	Label1-LabelBase
	call	Print_Label

	movf	PORTB,W
	movwf	oldb
	
	movlw	2
	movwf	brightness
	movlw	1
	movwf	direction

main_loop
	movf	TMR0,W
	subwf	brightness,W
	btfss	STATUS,Z
	goto	not_equal

	bsf		PORTB,1
; we havent reached the midpoint yet	
not_equal
	movf	PORTB,W
	movwf	newb
	movf	oldb,W
	xorwf	newb,W
	andwf	newb,W
	movwf	temp

	movf	newb,W
	movwf	oldb

	bcf		PORTB,1
	
	btfsc	temp,0	; test RB0 to see if it has changed from low to high
	goto	zero_crossing	
	

	goto main_loop

zero_crossing
	movf	TMR0,W
	movwf	period
	movlw	0xf0
	movwf	TMR0
	
	movlw   1
	addwf   count_low,F
	btfss   STATUS,C   
	goto    no_carry   
	            
	addwf   count,F

no_carry

	bcf		PORTB,1
	
	movf	direction,W
	addwf	brightness,F
	btfsc	STATUS,Z
	goto	dir_positive
	
	movf	brightness,W
	sublw	0x90
	btfsc	STATUS,Z
	goto	dir_negative
	goto	nothing
	
dir_positive
	movlw	1
	movwf	direction
	goto nothing
	
dir_negative
	movlw	0xff
	movwf	direction

nothing

	incf	ipc,F

	movlw	0x05
	call	LCD_SDDAM
		

;	movf	count,W
;	call	LCD_DISPNUM

;	movlw	' '
;	call	LCD_PUTCHAR

	movf	count_low,W
	call	LCD_DISPNUM

;	movlw	' '
;	call	LCD_PUTCHAR

;	movf	ipc,W
;	call	LCD_DISPNUM

	movlw	' '
	call	LCD_PUTCHAR

	movf	period,W
	call	LCD_DISPNUM

	goto	main_loop



	return	                
	                

org 0x300

Print_Label
	movwf	table_index
	call	LabelTable
	xorlw	0
	btfsc	STATUS,Z
	goto	Print_End
	call	LCD_PUTCHAR
	movf	table_index,W
	addlw 	1
	goto	Print_Label
Print_End
	return

; writes the value in W to the display in hex
LCD_DISPNUM
	movwf	numdisp_temp
	swapf	numdisp_temp,W
	call	Num_Table
	call	LCD_PUTCHAR
	movf	numdisp_temp,W
	call	Num_Table
	call	LCD_PUTCHAR
	return

init_lcd
	clrf	PORTA
	clrf	PORTB

	movlw	0x28
	call LCD_PUTCMD

	movlw	0x00
	call LCD_DMODE
	call LCD_CLEAR
	movlw	0x04
	call	LCD_DMODE
	movlw	0x02
	call	LCD_EMODE	
	
	return

LCD_SDDAM
	iorlw	0x80
	call	LCD_PUTCMD
	return

LCD_CLEAR
	movlw	0x01
	call	LCD_PUTCMD

LCD_EMODE
	andlw	0x03
	iorlw	0x04
	call	LCD_PUTCMD
	return

LCD_DMODE
	andlw	0x07
	iorlw	0x08
	call	LCD_PUTCMD
	return

LCD_BUSY
	movlw	0x0f
	andwf	PORTB,F
	bsf	STATUS,RP0
	movlw	b'11110000'
	iorwf	TRISB,F
	bcf	STATUS,RP0
	bcf	HD_RS
	bsf	HD_RW
	bsf	HD_E
	movf	PORTB,W
	bcf	HD_E
	andlw 0xf0
	movwf	temp
	bsf HD_E
	bcf	HD_E
	swapf	PORTB,W
	andlw 0x0f
	iorwf	temp,0xF
	btfsc	temp, 7
	goto	LCD_BUSY
	bsf	STATUS,RP0
	movlw	b'00001111'
	andwf	TRISB,F
	bcf	STATUS,RP0
	return
	

; put the command in A
LCD_PUTCMD
	movwf	lcd_temp
	call	LCD_BUSY
	movlw	0x0f
	andwf	PORTB,F
	movf	lcd_temp, W
	andlw	0xf0
	iorwf	PORTB,F
	bcf	HD_RW
	bcf	HD_RS
	bsf	HD_E
	bcf	HD_E
	movlw	0x0f
	andwf	PORTB,F
	swapf	lcd_temp, W
	andlw   0xf0
	iorwf	PORTB,F
	bsf	HD_E
	bcf HD_E
	return

LCD_PUTCHAR
	movwf	lcd_temp
	call	LCD_BUSY
	movlw	0x0f
	andwf	PORTB,F
	movf	lcd_temp,W
	andlw	0xf0
	iorwf	PORTB,F
	bcf	HD_RW
	bsf	HD_RS
	bsf	HD_E
	bcf	HD_E
	movlw	0x0f
	andwf	PORTB,F
	swapf	lcd_temp, W
	andlw	0xf0
	iorwf	PORTB,F
	bsf	HD_RS
	bsf	HD_E
	bcf	HD_E
	return






;
; delay subroutine
; input: delay count in W
;
; inner loop duration approx:
; 5*256+3 = 1283 cycles ->
; 1.28ms with 4MHz crystal (1MHz instruction time)
;
delay	movwf	delay_cnt1
	clrf	delay_cnt2
delay_a	nop
	nop
	incfsz	delay_cnt2,F
	goto	delay_a
	decfsz	delay_cnt1,F
	goto	delay_a
	return




	end
