; Multi-Channel Dimmer Controller

;
; Listens on a 232-bus for commands to execute
;
; Packet format
; BYTE 1
;  1 i3 i2 i1 i0 l2 l1 l0
;
;   i3-i0	DEVICE ID	(Overall device ID)
;   l2-l0	LIGHT ID	(Number of the light within this device)
;
; BYTE 2
;  0 b7-b0
;
;   b7-b0	BRIGHTNESS	(7 bit brightness value)
;




;
; define PIC device type
;
;	device	pic16f84

;
; define config fuses
;
;	config	CP=off,WDT=off,PWRT=off,OSC=rc
	__config _wdt_off & _rc_osc

;
; include PIC register definitions, and some macros
;
;	include "pic16c84.h"
;	include "picmac.h"

	include "../include/p16f84.inc"
	include "../include/picmac_gpasm.h"

;
;                        -----------
;                       |     U     |
;               HD_E  --|RA2     RA1|-- HD_R/W
;                     --|RA3     RA0|-- HD_RS
;                     --|RA4        |
;                       |           | 
;                       |           |
;                       |           |
;        ZC detector  --|RB0     RB7|-- D7
;     Light 1 triac   --|RB1     RB6|-- Light 6 triac
;     Light 2 triac   --|RB2     RB5|-- Light 5 triac
;     Light 3 triac   --|RB3     RB4|-- Light 4 triac
;                       |           |
;                        -----------



;
; bit definitions for two LEDs connected to port A bits 0 and 1
; bit masks can be computed from bit numbers with the left shift operator.
;

#define DEVICE_ID 1

#define HD_RS PORTA, 0
#define HD_RW PORTA, 1
#define HD_E PORTA, 2

#define LED PORTB, 1

#define SYNC_CLOCK PORTB, 7
#define SYNC_DATA PORTA,3
#define SYNC_RECEIVED ipc, 0
;
; define some register file variables
;

	SEGU data

	org	0x0c

temp DS 1
temp_w DS 1
temp_s DS 1

count_low DS 1
count DS 1

period DS 1
oldb DS 1
newb DS 1

brightness DS 1
brightness2 DS 1
brightness3 DS 1
brightness4 DS 1
brightness5 DS 1
brightness6 DS 1
direction DS 1

	SEG code

;
; code start
;
	org	0
	goto start

	org 4
	save_w_stat

	movf	TMR0,W
	movwf	period
	movlw	0xE6
	movwf	TMR0
	
	decf	brightness,F
	decf	brightness2,F
	decf	brightness3,F
	decf	brightness4,F
	decf	brightness5,F
	decf	brightness6,F

	bcf		INTCON,INTF

	restore_w_stat
 	retfie


#include "../include/number_table.asm"

LabelTable
	addwf	PCL,F
LabelBase
Label0
	retlw 'L'
	retlw 'C'
	retlw 'D'
	retlw ' '
	retlw ' '
	retlw 0
Label1
	retlw 'T'
	retlw 'e'
	retlw 's'
	retlw 't'
	retlw ' '
	retlw 0
LabelTable_End
	retlw 0

	
;org 0x100
start
	movlw	0xff
	call	delay

	movlw	0x00
	movwf	PORTA	;initialize port A so that LEDs are off

	movlw	0x00
	movwf	PORTB
	bsf	LED

	bsf	STATUS,RP0			;register page 1
	movlw	B'00001000'			;a4 input, all others output
	movwf	TRISA				;other PORTA pins as inputs
	movlw	B'00000001'			;b0 input, all others output
	movwf	TRISB

	; set options
	; 7 : 1 PORTB pull-ups are disabled
	; 6 : 1 Interrupt on rising esdge of RB0
	; 5 : 0 TMR0 on internal clock
	; 4 : 1 increment on high to low transition at RA4 for external clock
	; 3 : 0 prescaler assigned to TMR0
	; 2-0 : 101  1:64 timer prescale
	movlw	B'11010101'
	movwf	OPTION_REG

	bcf	STATUS,RP0			;register page 0


; turn off timer overflow interrupt	
	bcf INTCON,T0IE

; turn on RB0 edge interrupt
	bsf INTCON,INTE

	bcf	INTCON, RBIE

; globally enable interrupts
	bsf	INTCON,GIE

	movlw	50
	call delay

	call init_lcd

	movlw	Label0-LabelBase
	call	Print_Label

	movlw	0x40
	call	LCD_SDDAM

	movlw	Label1-LabelBase
	call	Print_Label

	movf	PORTB,W
	movwf	oldb
	
	movlw	2
	movwf	brightness
	movlw	0x20
	movwf	brightness2
	movlw	0x50
	movwf	brightness3
	movlw	0x80
	movwf	brightness4
	movlw	0xa0
	movwf	brightness5
	movlw	0xd0
	movwf	brightness6
	movlw	1
	movwf	direction

main_loop

	; check the first light
	movf	TMR0,W
	subwf	brightness,W
	btfsc	STATUS,Z
	bsf		PORTB,1
	
	bcf		PORTB,5

	; check to see if light 2 should be pulsed
	movf	TMR0,W
	subwf	brightness2,W
	btfsc	STATUS,Z
	bsf		PORTB,2
	
	bcf		PORTB,6

	; check to see if light 3 should be pulsed
	movf	TMR0,W
	subwf	brightness3,W
	btfsc	STATUS,Z
	bsf		PORTB,3
	
	bcf		PORTB,1

	; check to see if light 4 should be pulsed
	movf	TMR0,W
	subwf	brightness4,W
	btfsc	STATUS,Z
	bsf		PORTB,4
	
	bcf		PORTB,2

	; check to see if light 5 should be pulsed
	movf	TMR0,W
	subwf	brightness5,W
	btfsc	STATUS,Z
	bsf		PORTB,5
	
	bcf		PORTB,3

	; check to see if light 6 should be pulsed
	movf	TMR0,W
	subwf	brightness6,W
	btfsc	STATUS,Z
	bsf		PORTB,6
	
	bcf		PORTB,4
	

	goto	main_loop

	movf	PORTB,W
	movwf	newb
	movf	oldb,W
	xorwf	newb,W
	andwf	oldb,W
	movwf	temp

	movf	newb,W
	movwf	oldb

	; turn off all lights
	movlw	0
	movwf	PORTB
	
	btfsc	temp,0	; test RB0 to see if it has changed from low to high
	goto	zero_crossing	
	

	goto main_loop

zero_crossing
	movf	TMR0,W
	movwf	period
	movlw	0xE6
	movwf	TMR0
	
	incf	brightness,F
	incf	brightness2,F
	incf	brightness3,F
	incf	brightness4,F
	incf	brightness5,F
	incf	brightness6,F

; write debugging info out to screen
nothing
	goto	main_loop

	movlw	0x05
	call	LCD_SDDAM
		
	movf	count_low,W
	call	LCD_DISPNUM

	movlw	' '
	call	LCD_PUTCHAR

	movf	period,W
	call	LCD_DISPNUM

	goto	main_loop



	org 0x300

#define TWOWIRE_CLK PORTB,6
#define TWOWIRE_DATA PORTB,7
#define BAUDRATE 19200

#include "../include/lcd_2wire.asm"

#include "../include/delay.asm"

	end
