; Multi-Channel Dimmer Controller

;
; Listens on a 232-bus for commands to execute
;
; Packet format
; BYTE 1
;  1 i3 i2 i1 i0 l2 l1 l0
;
;   i3-i0	DEVICE ID	(Overall device ID)
;   l2-l0	LIGHT ID	(Number of the light within this device)
;
; BYTE 2
;  0 b7-b0
;
;   b7-b0	BRIGHTNESS	(7 bit brightness value)
;




;
; define PIC device type
;
;	device	pic16f84

;
; define config fuses
;
;	config	CP=off,WDT=off,PWRT=off,OSC=rc
	__config _wdt_off & _rc_osc

;
; include PIC register definitions, and some macros
;
;	include "pic16c84.h"
;	include "picmac.h"

	include "../include/p16f84.inc"
	include "../include/picmac_gpasm.h"

#include "../include/uart_3110.h"

;
;                        -----------
;                       |     U     |
;            SPI_CLK  --|RA2     RA1|-- MAX3110_CS/
;            SPI_DIN  --|RA3     RA0|-- MAX3110_IRQ/
;           SPI_DOUT  --|RA4        |
;                       |           | 
;                       |           |
;                       |           |
;        ZC detector  --|RB0     RB7|--   
;     Light 1 triac   --|RB1     RB6|-- Light 6 triac
;     Light 2 triac   --|RB2     RB5|-- Light 5 triac
;     Light 3 triac   --|RB3     RB4|-- Light 4 triac
;                       |           |
;                        -----------



;
; bit definitions for two LEDs connected to port A bits 0 and 1
; bit masks can be computed from bit numbers with the left shift operator.
;

#define DEVICE_ID 0x04
#define PACKET_SIZE 3

#define MAX3110_IRQ PORTA,0
#define MAX3110_CS	PORTA,1

; configuration for the MAX3110 UART



;
; define some register file variables
;

	SEGU data

	org	0x0c

temp DS 1
temp_w DS 1
temp_s DS 1

count_low DS 1
count DS 1

period DS 1
oldb DS 1
newb DS 1

brightness DS 1
brightness2 DS 1
brightness3 DS 1
brightness4 DS 1
brightness5 DS 1
brightness6 DS 1

; latches for the brightness vars
ldim1 DS 1
ldim2 DS 1
ldim3 DS 1
ldim4 DS 1
ldim5 DS 1
ldim6 DS 1

config1		DS 1
config2		DS 1

direction DS 1

packet_count DS 1
packet_buf DS 3

temp1 DS 1
temp2 DS 1

	SEG code

;
; code start
;
	org	0
	goto start

	org 4
	save_w_stat

	movf	TMR0,W
	movwf	period
	movlw	0x00
	movwf	TMR0

	movf	ldim1,W
	movwf	brightness
	movf	ldim2,W
	movwf	brightness2
	movf	ldim3,W
	movwf	brightness3
	movf	ldim4,W
	movwf	brightness4
	movf	ldim5,W
	movwf	brightness5
	movf	ldim6,W
	movwf	brightness6

	btfss	config1,0
	goto	no_auto_inc
	decf	ldim1,F
	decf	ldim2,F
	decf	ldim3,F
	decf	ldim4,F
	decf	ldim5,F
	decf	ldim6,F
no_auto_inc	

	bcf		INTCON,INTF

	restore_w_stat
 	retfie


start
	movlw	0xff
	call	delay

	clrf	count

	clrf	PORTA
	clrf	PORTB	

	bsf	STATUS,RP0			;register page 1
	movlw	B'00010001'			;a4,a0 input, all others output
	movwf	TRISA				;other PORTA pins as inputs
	movlw	B'00000001'			;b0 input, all others output
	movwf	TRISB

	; set options
	; 7 : 1 PORTB pull-ups are disabled
	; 6 : 1 Interrupt on rising esdge of RB0
	; 5 : 0 TMR0 on internal clock
	; 4 : 1 increment on high to low transition at RA4 for external clock
	; 3 : 0 prescaler assigned to TMR0
	; 2-0 : 101  1:256 timer prescale
	movlw	B'11010111'
	movwf	OPTION_REG

	bcf	STATUS,RP0			;register page 0


; turn off timer overflow interrupt	
	bcf INTCON,T0IE

; turn on RB0 edge interrupt
	bsf INTCON,INTE

	bcf	INTCON, RBIE

; globally enable interrupts
	bsf	INTCON,GIE

#define MY_UART_CONFIG WRITE_CONFIGURATION | FIFO_INT_ENABLED | B57600 | FIFO_DISABLED | PARITY_ENABLED

	movlw	HIGH(MY_UART_CONFIG)
	movwf	uart_3110_temp
	movlw	LOW(MY_UART_CONFIG)
	movwf	uart_3110_temp2
	call	UART_232_INIT_PARM
	
	movlw	'J'
	call	UART_232_TX
	movlw	'J'
	call	UART_232_TX
	movlw	'P'
	call	UART_232_TX


	clrf	config1
	clrf	config2
	
	movlw	2
	movwf	ldim1
	movlw	0x20
	movwf	ldim2
	movlw	0x50
	movwf	ldim3
	movlw	0x80
	movwf	ldim4
	movlw	0xa0
	movwf	ldim5
	movlw	0xd0
	movwf	ldim6
main_loop_packet_clear
	clrf	packet_count

main_loop

	; check the first light
	movf	TMR0,W
	subwf	brightness,W
	btfsc	STATUS,Z
	bsf		PORTB,1
	btfss	STATUS,Z
	bcf		PORTB,1
	
	; check to see if light 2 should be pulsed
	movf	TMR0,W
	subwf	brightness2,W
	btfsc	STATUS,Z
	bsf		PORTB,2
	btfss	STATUS,Z
	bcf		PORTB,2
	

	; check to see if light 3 should be pulsed
	movf	TMR0,W
	subwf	brightness3,W
	btfsc	STATUS,Z
	bsf		PORTB,3
	btfss	STATUS,Z
	bcf		PORTB,3	

	; check to see if light 4 should be pulsed
	movf	TMR0,W
	subwf	brightness4,W
	btfsc	STATUS,Z
	bsf		PORTB,4
	btfss	STATUS,Z
	bcf		PORTB,4
	

	; check to see if light 5 should be pulsed
	movf	TMR0,W
	subwf	brightness5,W
	btfsc	STATUS,Z
	bsf		PORTB,5
	btfss	STATUS,Z	
	bcf		PORTB,5

	; check to see if light 6 should be pulsed
	movf	TMR0,W
	subwf	brightness6,W
	btfsc	STATUS,Z
	bsf		PORTB,6
	btfss	STATUS,Z	
	bcf		PORTB,6
	
	btfsc	MAX3110_IRQ
	goto	main_loop

	; here means the UART is signalling us with an interrupt...
;read byte  6(9)+UART_232_RX=150(154)

	call	UART_232_RX	; read in the byte  
	addlw	0			; check to see if there actually is a byte
	btfsc	STATUS,Z	
	goto	main_loop	


;	movf	spi_low,W
;	movwf	temp1
;	movf	spi_high,W
;	movwf	temp2

;	movf	temp2,W
;	call	UART_232_TX
	
;	movf	temp1,W
;	call	UART_232_TX

;	movlw	'R'
;	call	UART_232_TX

;	movf	temp1,W
;	movwf	spi_low
;	movf	temp2,W
;	movwf	spi_high

	call	UART_232_PARITY_CHECK
	addlw	0
	btfss	STATUS,Z
	goto	main_loop_packet_clear
	
	movf	packet_count,F
	btfss	STATUS,Z
	goto	add_to_packet
	
	btfss	spi_low,7
	goto	main_loop

; here we are receiving the first byte of a new packet	
address_byte ; (6)
	movlw	1
	movwf	packet_count
	movf	spi_low,W
	movwf	packet_buf
	
;	movlw	'A'
;	call	UART_232_TX
	goto	main_loop

; here we are adding to an already completed packet	
add_to_packet  ; (13)
	movlw	packet_buf
	movwf	FSR
	movf	packet_count,W
	addwf	FSR,F
	movf	spi_low,W
	movwf	IND0
	

;	movlw	'I'
;	call	UART_232_TX

	incf	packet_count,F
	movf	packet_count,W
	sublw	PACKET_SIZE

	btfss	STATUS,Z
	goto	main_loop
	
packet_is_complete ; 7,20
;	movlw	'C'
;	call	UART_232_TX
	clrf	packet_count

	; here we see if the packet belongs to us, if it does
	; then we should act on it
	movf	packet_buf,W
	andlw	B'01111100'
	sublw	DEVICE_ID
	btfss	STATUS,Z
	goto	main_loop	
	
;	movlw	'M'
;	call	UART_232_TX

	; the packet is for us...
	movlw	ldim1
	movwf	FSR
	
	movlw	B'00000011'
	andwf	packet_buf,F
	bcf		STATUS,C
	rlf		packet_buf,W
	addwf	FSR,F

;	movf	FSR,W
;	call	UART_232_TX
	
	movf	packet_buf+1,W
	movwf	IND0

;	call	UART_232_TX

	incf	FSR,F

	movf	packet_buf+2,W
	movwf	IND0
	
;	call	UART_232_TX
	
	goto	main_loop	
	
	org 0x300

#define SPI_CLK	PORTA,2
#define SPI_DIN PORTA,3
#define SPI_DOUT PORTA,4
#define BAUD_RATE_3110 B19200

#include "../include/uart_3110.asm"

#include "../include/delay.asm"
	end
